﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace SampleApplication
{
    public partial class Form1 : Form
    {
        internal const float PointSize = 7.5f;

        private LayerOverlay pointOverlay;

        private Feature leftPoint;

        private HighlightingPointStyle pointStyle;

        public Form1()
        {
            InitializeComponent();
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.SimpleColors.White);
            winformsMap1.CurrentExtent = new RectangleShape(-85, 90.13, 87.28, -48.04);

            InitializeAreaLayer();
            InitializeLayer();

            winformsMap1.Refresh();
        }

        private void InitializeAreaLayer()
        {
            var overlay = new LayerOverlay();
            var layer = new InMemoryFeatureLayer();
            ZoomLevel level = layer.ZoomLevelSet.ZoomLevel01;
            level.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.SimpleColors.Red));
            level.DefaultAreaStyle = new AreaStyle(new GeoSolidBrush(GeoColor.SimpleColors.Yellow));
            level.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var shape = new EllipseShape(new PointShape(0, 0), 20, 20);
            layer.InternalFeatures.Add(new Feature(shape));
            layer.BuildIndex();
            layer.DrawingQuality = DrawingQuality.HighSpeed;
            overlay.Layers.Add(layer);
            this.winformsMap1.Overlays.Add(overlay);
        }

        private void InitializeLayer()
        {
            this.pointOverlay = new LayerOverlay();
            var layer = new InMemoryFeatureLayer();
            this.leftPoint = AddPoint(layer, 0, 0);
            AddPoint(layer, 1, 0);
            ZoomLevel level = layer.ZoomLevelSet.ZoomLevel01;
            level.DefaultPointStyle = (this.pointStyle = new HighlightingPointStyle(this.leftPoint));
            level.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            const int width = 100;
            for (int x = 0; x <= width; x++)
            {
                for (int y = 0; y <= width; y++)
                {
                    AddPoint(layer, -40 + (80 * (double)x) / width, -40 + (80 * (double)y) / width);
                }
            }

            layer.DrawingQuality = DrawingQuality.HighSpeed;
            layer.BuildIndex();
            this.pointOverlay.Layers.Add(layer);
            this.winformsMap1.Overlays.Add(this.pointOverlay);
        }

        private static Feature AddPoint(InMemoryFeatureLayer layer, double x, double y)
        {
            var shape = new PointShape(x, y);
            var feature = new Feature(shape);
            layer.InternalFeatures.Add(feature);
            return feature;
        }

        private void button2_Click(object sender, System.EventArgs e)
        {
            MeasureTime(
                () =>
                {
                    this.pointStyle.IsHighlightActive = !this.pointStyle.IsHighlightActive;
                    this.winformsMap1.Refresh(this.pointOverlay);
                });
        }

        private async void MeasureTime(Action action)
        {
            Stopwatch watch = Stopwatch.StartNew();
            action();
            watch.Stop();
            this.label1.Text = string.Format("Elapsed time: {0} s", (double)watch.ElapsedMilliseconds/1000);
            this.label1.Left = this.panel1.Width - 8 - this.label1.Width;
            this.label1.Visible = true;
            for (int i = 1; i <= 3; i++)
            {
                await Task.Delay(500);
                this.label1.Visible = false;
                await Task.Delay(500);
                this.label1.Visible = true;
            }
        }
    }
}
