﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using ThinkGeo.MapSuite.Core;

namespace SampleApplication
{
    public sealed class HighlightingPointStyle : PointStyle
    {
        private readonly Feature highightedFeature;

        public bool IsHighlightActive { get; set; }

        public HighlightingPointStyle(Feature highightedFeature)
        {
            this.highightedFeature = highightedFeature;
            SymbolSolidBrush = new GeoSolidBrush(GeoColor.SimpleColors.Black);
            SymbolSize = 7.5f;
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            foreach (IGrouping<bool, Feature> featureGroup in features.GroupBy(f => IsHighlightActive && f == highightedFeature).OrderBy(g => g.Key))
            {
                new PointStyle
                {
                    SymbolSolidBrush = new GeoSolidBrush(featureGroup.Key ? GeoColor.SimpleColors.Red : GeoColor.StandardColors.Gray),
                    SymbolSize = 7.5f
                }.Draw(featureGroup, canvas, labelsInThisLayer, labelsInAllLayers);
            }
        }
    }
}
